package hirondelle.web4jtools.codegenerator.field;

import hirondelle.web4j.action.ActionTemplateListAndEdit;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.database.DAOException;
import hirondelle.web4j.database.DuplicateException;
import hirondelle.web4j.model.Id;
import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.model.ModelFromRequest;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.util.Util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

/**
* Edit the fields used by the feature.
* 
* <P>The order of how items are added will be retained in generated code.
*/
public final class FieldAction extends ActionTemplateListAndEdit {

  /** Constructor.  */
  public FieldAction(RequestParser aRequestParser){
    super(FORWARD, REDIRECT_TO_LISTING, aRequestParser);
  }
  
  public static final RequestParameter ID = RequestParameter.withLengthCheck("Id");
  public static final RequestParameter NAME = RequestParameter.withLengthCheck("Name");
  public static final RequestParameter DESCRIPTION = RequestParameter.withLengthCheck("Description");
  public static final RequestParameter IS_REQUIRED = RequestParameter.withLengthCheck("IsRequired");
  public static final RequestParameter TYPE = RequestParameter.withLengthCheck("Type");
  public static final RequestParameter CONTROL_STYLE = RequestParameter.withLengthCheck("ControlStyle");
  public static final RequestParameter IS_PRIMARY_KEY = RequestParameter.withRegexCheck("IsPrimaryKey", "(true|false)");
  public static final RequestParameter MINIMUM = RequestParameter.withLengthCheck("Minimum");
  public static final RequestParameter MAXIMUM = RequestParameter.withLengthCheck("Maximum");
  public static final RequestParameter ERROR_MESSAGE = RequestParameter.withLengthCheck("ErrorMessage");
  public static final RequestParameter IS_HARD_VALIDATED_FOR_LENGTH = RequestParameter.withRegexCheck("IsHardValidatedForLength", "(true|false)");
  public static final RequestParameter HARD_VALIDATON_PATTERN = RequestParameter.withLengthCheck("HardValidationPattern");
  public static final RequestParameter CHECK_PATTERN = RequestParameter.withLengthCheck("CheckPattern");
  public static final RequestParameter CHECK_EMAIL = RequestParameter.withLengthCheck("CheckEmail");
  public static final RequestParameter CHECK_SPAM = RequestParameter.withLengthCheck("CheckSpam");
  public static final RequestParameter IS_ORDER_BY_FIELD = RequestParameter.withRegexCheck("IsOrderByField", "(true|false)");
  public static final RequestParameter IS_DESC_ORDER = RequestParameter.withRegexCheck("IsDescendingOrder", "(true|false)");
  public static final RequestParameter NUM_DECIMALS = RequestParameter.withLengthCheck("NumDecimals");
  public static final RequestParameter CODE_TABLE = RequestParameter.withLengthCheck("CodeTable");

  /** Key for storing fields in session scope. */
  public static final String FIELDS_KEY = "fields";
  
  /** List all Fields, in the same order as created.  */
  protected void doList() throws DAOException {
    addToRequest(ITEMS_FOR_LISTING, list());
  }

  /** Ensure user input can build a {@link Field}.  */
  protected void validateUserInput() {
    try {
      ModelFromRequest builder = new ModelFromRequest(getRequestParser());
      fField = builder.build(
        Field.class, ID, NAME, DESCRIPTION, IS_REQUIRED, TYPE, CONTROL_STYLE, IS_PRIMARY_KEY, MINIMUM, 
        MAXIMUM, NUM_DECIMALS, ERROR_MESSAGE, IS_HARD_VALIDATED_FOR_LENGTH, HARD_VALIDATON_PATTERN, 
        CHECK_PATTERN, CHECK_EMAIL, CHECK_SPAM, IS_ORDER_BY_FIELD, IS_DESC_ORDER, CODE_TABLE
      );
    }
    catch (ModelCtorException ex){
      addError(ex);
    }    
  }
  
  /** Add a new {@link Field}.  */
  protected void attemptAdd() throws DAOException {
    try {
      add(fField);
      addMessage("_1_ added successfully.", fField.getName());
    }
    catch(DuplicateException ex){
      addError("Field Name already taken. Please use another name. Must be unique.");  
    }
  }
  
  /** Fetch an existing {@link Field} in order to edit it.  */
  protected void attemptFetchForChange() throws DAOException {
    Field field = fetch(getIdParam(ID));
    if( field != null ){
      addToRequest(ITEM_FOR_EDIT, field);
    }
  }
  
  /** Apply an edit to an existing {@link Field}.  */
  protected void attemptChange() throws DAOException {
    boolean success = change(fField);
    if (success){
      addMessage("_1_ changed successfully.", fField.getName());
    }
    else {
      addError("No update occurred. Item deleted?");
    }
  }
  
  /** Ensure the {@link Field} of a given id does not exist. */
  protected void attemptDelete() throws DAOException {
    Id id = getIdParam(ID);
    fLogger.fine("Deleting with id " + id);
    delete(id);
    addMessage("Item deleted successfully.");
  }
  
  // PRIVATE //
  private Field fField;
  private static final ResponsePage FORWARD =  new ResponsePage("Fields", "view.jsp", FieldAction.class);
  private static final ResponsePage REDIRECT_TO_LISTING = new ResponsePage("FieldAction.do?Operation=List");
  private static final Logger fLogger = Util.getLogger(FieldAction.class);
  
  private List<Field> list() {
    List<Field> result = new ArrayList<Field>();
    Object object = getFromSession(FIELDS_KEY);
    if ( object != null ) {
      result = (List<Field>)object;
    }
    return result;
  }

  private Field fetch(Id aFieldId) {
    Field result = null;
    List<Field> fields = list();
    if ( fields == null ) {
      addError("Cannot fetch item. No Fields found in session.");
    }
    else {
      Integer id = Integer.valueOf(aFieldId.toString());
      if ( id > fields.size() -1) {
        addError("Cannot find Field of index " + id);
      }
      else {
        result = fields.get(id);
      }
    }
    return result;
  }
  
  private void add(Field aField) throws DuplicateException {
    if ( contains(aField.getName()) ) {
      RuntimeException ex = new RuntimeException();
      throw new DuplicateException("That field name is already taken. Please select another name.", ex);
    }
    List<Field> list = list();
    list.add(aField); //adds to the end of the list
    applyUpdate(list);
  }

  private void applyUpdate(List<Field> aList) {
    fLogger.fine("Applying update to fields stored in session.");
    addToSession(FIELDS_KEY, aList);
  }

  private boolean contains(String aFieldName){
    boolean result = false;
    List<Field> fields = list();
    for(Field field : fields ){
      if ( field.getName().equalsIgnoreCase(aFieldName)) {
        result = true;
        break;
      }
    }
    return result;
  }
  
  private void  delete(Id aFieldId){
    Integer id = Integer.valueOf(aFieldId.toString());
    List<Field> fields = list();
    if ( fields == null ) {
      addError("No Fields in session. Cannot delete item.");
    }
    if ( id > fields.size() - 1 ) {
      addError("Cannot find item using index : " + id);
    }
    else {
      fLogger.fine("Removing item using id " + id);
      fields.remove(id.intValue()); //must use int, not Integer!!! overloaded with Object
      applyUpdate(fields);
    }
  }
  
  private boolean change(Field aReplacementField) throws DuplicateException {
    boolean result = false;
    Integer id = Integer.valueOf(aReplacementField.getId().toString());
    //replace it with aReplacementField
    List<Field> fields = list();
    Iterator<Field> iter = fields.iterator();
    int index = 0;
    while ( iter.hasNext() ) {
      if ( id == index ){
        fields.set(index, aReplacementField);
        result = true;
        break;
      }
      ++index;
    }
    return result;
  }
}
