package hirondelle.web4jtools.logview.directories;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;

import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.security.SafeText;
import hirondelle.web4j.util.Util;
import hirondelle.web4j.util.WebUtil;

/**
* Save and fetch the current {@link LogInfo}.
* 
* <P>This implementation does not store items in a database. Rather, the following 
* mechanism is used : 
*<ul>
* <li>upon startup, store a default {@link LogInfo} in application scope, using default settings in <tt>web.xml</tt>.
* <li>allow the user to override the default settings, and store an 'override' {@link LogInfo} in <em>session</em> scope.
*</ul> 
*/
public final class LogInfoDAO {

  /** 
  * Read in config from web.xml.  
  * Called only upon startup. Place a default {@link LogInfo} object in application scope. 
  */
  public static void readConfig(ServletConfig aConfig) {
    try {
      LogInfo defaultDirInfo = new LogInfo(
        get(LogInfoAction.PROJECT_NAME, aConfig), 
        get(LogInfoAction.APP_LOGGING_DIR, aConfig), 
        get(LogInfoAction.SERVER_LOGGING_DIR, aConfig), 
        get(LogInfoAction.SERVER_LOG_FILE_STARTS_WITH, aConfig)
      );
      aConfig.getServletContext().setAttribute(LOG_INFO_KEY, defaultDirInfo);
    }
    catch (ModelCtorException ex) {
      throw new RuntimeException(ex);
    }
  }
  
  /** Full constructor  */
  public LogInfoDAO(HttpServletRequest aRequest) {
    fRequest = aRequest;
  }
  
  /** Key under which {@link LogInfo} objects are stored (in both application and session scopes.) */
  public static final String LOG_INFO_KEY = "logInfo";

  /**
  * Fetch a {@link LogInfo} from either session scope (preferred) or application scope (default).
  */
  public LogInfo fetch(){
    return (LogInfo)WebUtil.findAttribute(LOG_INFO_KEY, fRequest);
  }
  
  /** Save an 'override' {@link LogInfo} into session scope.  */
  public void save(LogInfo aDirInfo){
    fRequest.getSession(true).setAttribute(LOG_INFO_KEY, aDirInfo); //for later lookup
  }

  // PRIVATE //
  private HttpServletRequest fRequest;
  
  private static SafeText get(RequestParameter aRequestParameter, ServletConfig aConfig){
    String value = aConfig.getInitParameter(aRequestParameter.getName());
    if ( ! Util.textHasContent(value) ) {
      String message = "In web.xml, please specify a value for parameter named " + aRequestParameter;
      throw new RuntimeException(message);
    }
    return new SafeText(value);
  }
}
