package hirondelle.web4jtools.logview.parsedview;

import hirondelle.web4j.model.ModelUtil;
import hirondelle.web4j.security.SafeText;
import java.util.Date;
import java.util.logging.Level;
import hirondelle.web4j.util.Util;

/** 
* Model Object for a parsed Logger Record.
* 
* <P>A parsed logger record can be filtered and sorted according to various criteria, 
* making it easier to find needed information.
* 
* <P>This class is mutable.
*/
public final class LoggerRecord {

  /**
  * Full constructor.
  *  
  * @param aLevel Logging level (required).
  * @param aDate Date of the logging message (required).
  * @param aLogger Logger/package name (required).
  * @param aMethod name of the method emitting the message (optional).
  * @param aMessage Text of the logging message (required).
  */
  public LoggerRecord(
   String aLevel, 
   Date aDate, 
   String aLogger, 
   String aMethod,
   String aMessage
  ) {
    fLevel = Util.textHasContent(aLevel) ? Level.parse(aLevel) : null;
    fDate = aDate.getTime(); //defensive form
    fLogger = new SafeText(aLogger);
    fMethod = new SafeText(aMethod);
    fMessage = new SafeText(aMessage);
  }

  /**
  * Alternate constructor.
  * 
  * Useful for building a partial log from the first line of a record. 
  */
  public LoggerRecord(Date aDate, String aLogger, String aMethod) {
    fDate = aDate.getTime();
    fLogger = new SafeText(aLogger);
    fMethod = new SafeText(aMethod);
  }
  
  /** Set the level for this record. */
  public void addLevel(String aLevel) {
    fLevel = Util.textHasContent(aLevel) ? Level.parse(aLevel) : null;
  }
  
  /** Set the log message for this record. */
  public void addMessage(String aMessage) {
    fMessage = new SafeText(aMessage);
  }
  
  public Level getLevel() { return fLevel; }  
  public Date getDate() { return new Date(fDate); }
  public SafeText getLogger() { return fLogger; }  
  public SafeText getMethod() { return fMethod; }
  public SafeText getMessage() { return fMessage; }
  
  @Override public String toString(){
    return ModelUtil.toStringFor(this);
  }
  
  @Override public boolean equals(Object aThat){
    Boolean result = ModelUtil.quickEquals(this, aThat);
    if ( result == null ) {
      LoggerRecord that = (LoggerRecord)aThat;
      result = ModelUtil.equalsFor(this.getSignificantFields(), that.getSignificantFields());
    }
    return result;
  }
  
  @Override public int hashCode(){
    //cannot cache - mutable
    return ModelUtil.hashCodeFor(getSignificantFields());
  }

  // PRIVATE 
  private Level fLevel;
  private final Long fDate;
  private final SafeText fLogger;
  private final SafeText fMethod;
  private SafeText fMessage;

  private Object[] getSignificantFields(){
    return new Object[] {fLevel, fDate, fLogger, fMethod, fMessage};
  }
}