package hirondelle.web4jtools.logview.simpleview;

import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.model.Check;
import static hirondelle.web4j.util.Consts.FAILS;
import hirondelle.web4j.model.ModelUtil;
import hirondelle.web4j.util.Util;
import hirondelle.web4j.security.SafeText;

/**
* Model Object for the criteria entered by the user for viewing log files as simple text, 
* in an unparsed form. 
*/
public final class SimpleCriteria {
  
  /**
  * Full constructor. 
  * 
  * @param aLogFor toggle between application logs and server logs (required). See {@link LogFor}.
  * @param aSection either the first or last section of a log file; required only if <tt>aNumLines</tt> is specified. See {@link Section}.
  * @param aNumLines number of lines to display; required only if <tt>aSection</tt> is specified; range <tt>1..10,000</tt>.
  * @param aRefreshRate frequency in seconds in which to refresh the listing; range <tt>1..600</tt>.
  */
  public SimpleCriteria(SafeText aLogFor, SafeText aSection, Integer aNumLines, Integer aRefreshRate) throws ModelCtorException {
    fLogFor = Util.textHasContent(aLogFor) ? LogFor.valueOf(aLogFor.getRawString()) : null;
    fSection = Util.textHasContent(aSection)? Section.valueOf(aSection.getRawString()) : null;
    fNumLines = aNumLines;
    fRefreshRate = aRefreshRate;
    validateState();
  }
  
  Integer getNumLines(){ return fNumLines; }
  Integer getRefreshRate(){ return fRefreshRate; }
  Section getSection(){ return fSection; } 
  LogFor getLogFor() { return fLogFor; }

 @Override public String toString(){
    return ModelUtil.toStringFor(this);
  }
  
  @Override public  boolean equals(Object aThat){
    Boolean result = ModelUtil.quickEquals(this, aThat);
    if ( result ==  null ) {
      SimpleCriteria that = (SimpleCriteria) aThat;
      result = ModelUtil.equalsFor(this.getSignificantFields(), that.getSignificantFields());
    }
    return result;
  }
  
  @Override public int hashCode(){
     return ModelUtil.hashCodeFor(getSignificantFields());
  }  
  
  // PRIVATE //
  private Integer fNumLines;
  private Integer fRefreshRate;
  private Section fSection;
  private LogFor fLogFor;
  
  private void validateState() throws ModelCtorException {
    ModelCtorException ex = new ModelCtorException();
    if ( FAILS == Check.required(fLogFor) ) {
      ex.add("You must select Application or Server.");
    }
    if ( FAILS == Check.optional(fNumLines, Check.range(1,10000)) ) {
      ex.add("Num Lines is optional, 1..10,000.");
    }
    if ( FAILS == Check.optional(fRefreshRate, Check.range(1,600)) ) {
      ex.add("Refresh Rate is optional, 1..600 seconds.");
    }
    if ( fNumLines == null && fSection != null ) {
      ex.add("Num Lines must be specified.");
    }
    if ( fNumLines != null && fSection == null ) {
      ex.add("Section must be specified.");
    }
    if ( ! ex.isEmpty() ) throw ex;    
  }
  
  private Object[] getSignificantFields(){
    return new Object[]{fLogFor, fNumLines, fRefreshRate, fSection};
  }
}
