package hirondelle.web4jtools.metrics.base;

import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.security.SafeText;
import hirondelle.web4j.util.Util;
import hirondelle.web4j.util.WebUtil;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;

/**
* Save and fetch the current {@link BaseInfo}.
* 
* <P>This implementation does not store items in a database. Rather, the following 
* mechanism is used : 
*<ul>
* <li>upon startup, store a default {@link BaseInfo} in application scope, using default settings in <tt>web.xml</tt>.
* <li>allow the user to override the default settings, and store an 'override' {@link BaseInfo} in <em>session</em> scope.
*</ul> 
*/
public final class BaseInfoDAO {

  /** 
  * Read in config from web.xml.  
  * Called only upon startup. Place a default {@link BaseInfo} object in application scope. 
  */
  public static void readConfig(ServletConfig aConfig) {
    try {
      BaseInfo defaultBaseInfo = new BaseInfo(
        get(BaseInfoAction.PROJECT_NAME, aConfig), 
        get(BaseInfoAction.BASE_DIRECTORY, aConfig),
        get(BaseInfoAction.BASE_URI_FOR_FETCHING_IMAGES, aConfig)
      );
      aConfig.getServletContext().setAttribute(BASE_INFO_KEY, defaultBaseInfo);
    }
    catch (ModelCtorException ex) {
      throw new RuntimeException(ex);
    }
  }
    
  /** Full constructor  */
  public BaseInfoDAO(HttpServletRequest aRequest) {
    fRequest = aRequest;
  }
  
  /** Key under which {@link BaseInfo} objects are stored (in both application and session scopes.) */
  public static final String BASE_INFO_KEY = "baseInfo";

  /**
  * Fetch a {@link BaseInfo} from either session scope (preferred) or application scope (default).
  */
  public BaseInfo fetch(){
    return (BaseInfo)WebUtil.findAttribute(BASE_INFO_KEY, fRequest);
  }
  
  /** Save an 'override' {@link BaseInfo} into session scope.  */
  public void save(BaseInfo aDirInfo){
    fRequest.getSession(true).setAttribute(BASE_INFO_KEY, aDirInfo); //for later lookup
  }

  // PRIVATE //
  private HttpServletRequest fRequest;
  
  private static SafeText get(RequestParameter aRequestParameter, ServletConfig aConfig){
    String value = aConfig.getInitParameter(aRequestParameter.getName());
    if ( ! Util.textHasContent(value) ) {
      String message = "In web.xml, please specify a value for parameter named " + aRequestParameter;
      throw new RuntimeException(message);
    }
    return new SafeText(value);
  }
}
