package hirondelle.web4jtools.metrics.base;

import static hirondelle.web4j.util.Consts.NEW_LINE;
import static hirondelle.web4j.util.Consts.NOT_FOUND;
import hirondelle.web4j.model.ModelUtil;
import hirondelle.web4j.security.SafeText;
import hirondelle.web4j.util.Util;
import hirondelle.web4jtools.util.Ensure;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;
import java.util.logging.Logger;

import javax.servlet.ServletConfig;

/** Model Object for basic File Information. */
public final class FileInfo {

  /**
  * Read in config from web.xml.
  * 
  * <P>Called only during startup.
  */
  public static void readConfig(ServletConfig aConfig){
    String sourceFileExtensions = fetchConfigFor(SOURCE_FILE_EXTENSIONS, aConfig);
    setFileExtensions(sourceFileExtensions, fSourceFileExtensions);
    
    String imageFileExtensions = fetchConfigFor(IMAGE_FILE_EXTENSIONS, aConfig);
    setFileExtensions(imageFileExtensions, fImageFileExtensions);

    String markupFileExtensions = fetchConfigFor(MARKUP_FILE_EXTENSIONS, aConfig);
    setFileExtensions(markupFileExtensions, fMarkupFileExtensions);
    
    String ignorableFiles = fetchConfigFor(IGNORABLE_FILES, aConfig);
    setIgnorableFiles(ignorableFiles);
    
    fUNIT_TEST_FINGERPRINT = fetchConfigFor(UNIT_TEST_FINGERPRINT, aConfig);
  }
  
  /** Returned the list of ignorable files, as configured in <tt>web.xml</tt>. */
  static public String getIgnorableFiles(){
    return formattedList(fIgnorableFiles);
  }

  /** Return the list of extensions for source files, as configured in <tt>web.xml</tt>. */
  static public String getSourceFileExtensions() {
    return formattedList(fSourceFileExtensions);
  }
  
  /** Return the list of extensions for image files, as configured in <tt>web.xml</tt>. */
  static public String getImageFileExtensions() {
    return formattedList(fImageFileExtensions);
  }
  
  /** Return the list of extensions for markup files, as configured in <tt>web.xml</tt>. */
  static public String getMarkupFileExtensions() {
    return formattedList(fMarkupFileExtensions);
  }
  
  /** 
  * Determine if a file is an 'ignorable' file (such as a <tt>.class</tt> file, for instance.)
  * See {@link #getIgnorableFiles()}.  
  */
  public static boolean isIgnorable(File aFile) {
    boolean result = false;
    for(String ignorable : fIgnorableFiles){
      if ( aFile.getAbsolutePath().contains(ignorable) ) {
        result = true;
        break;
      }
    }
    return result;
  }
  
  /** Full constructor.   */
  public FileInfo(File aFile)  {
    fFile = aFile;
    if ( fFile.isDirectory() ) {
      throw new RuntimeException("File must be a file, not a directory.");
    }
    if ( isSourceFile() ) {
      examineLines();
    }
    if ( isJarFile() ) {
      examineJarSpecs();
    }
  }
  
  /** Return the absolute path name of the file. */
  public SafeText getName() { 
    return new SafeText(fFile.getAbsolutePath()); 
  }  
  
  /** Return the simple name of the file, without path information.  */
  public SafeText getSimpleName() {
    return new SafeText(fFile.getName());
  }
  
  /** Return the size of the file in bytes.  */
  public Long getSize() { 
    return fFile.length(); 
  }
  
  /** Return the date the file was last modified. */
  public Date getLastModified() { 
    return new Date(fFile.lastModified()); 
  }
  
  /** Return the file extension. */
  public SafeText getExtension() {
    SafeText result = new SafeText(""); //empty by default
    String name = getName().getRawString();
    int lastPeriod = name.lastIndexOf(".");
    if ( lastPeriod != NOT_FOUND ) {
      result = new SafeText(name.substring(lastPeriod));
    }
    return result;
  }
  
  /** 
  * Return true only if the extension matches one of the source file extensions configured in <tt>web.xml</tt>.
  * See {@link #getSourceFileExtensions()}. 
  */
  public Boolean isSourceFile() {
    boolean result = false;
    String extension = getExtension().getRawString();
    for (String sourceFileExtension : fSourceFileExtensions) {
      if ( extension.equalsIgnoreCase(sourceFileExtension) ) {
        result = true;
        break;
      }
    }
    return result;
  }
  
  /** 
  * Return true only if the extension matches one of the image file extensions configured in <tt>web.xml</tt>.
  * See {@link #getImageFileExtensions()}. 
  */
  public Boolean isImageFile(){
    boolean result = false;
    String extension = getExtension().getRawString();
    for (String imageFileExtension : fImageFileExtensions) {
      if ( extension.equalsIgnoreCase(imageFileExtension) ) {
        result = true;
        break;
      }
    }
    return result;
  }

  /** 
  * Return true only if the extension matches one of the markup file extensions configured in <tt>web.xml</tt>.
  * See {@link #getMarkupFileExtensions()}. 
  */
  public Boolean isMarkupFile(){
    boolean result = false;
    String extension = getExtension().getRawString();
    for (String markupFileExtension : fMarkupFileExtensions) {
      if ( extension.equalsIgnoreCase(markupFileExtension) ) {
        result = true;
        break;
      }
    }
    return result;
  }
  
  /** Return true only if the file extension is '.java' */
  public Boolean isJavaSourceFile() {
    return getExtension().getRawString().equalsIgnoreCase(".java");
  }
  
  /** Return true only if the file extension is '.class' */
  public Boolean isJavaClassFile() {
    return getExtension().getRawString().equalsIgnoreCase(".class");
  }
  
  /** Return true only if the file extension is '.jar' */
  public Boolean isJarFile(){
    return getExtension().getRawString().equalsIgnoreCase(".jar");
  }

  /** Return the number of lines (source files only)  */
  public Integer getNumLines() {
    return fNumLines;
  }
  
  /** Return the number of comments (java files only)  */
  public Integer getNumCommentLines(){
    return fNumCommentLines;
  }
  
  /** Return the percentage of comments as part of total lines (java files only)  */
  public Integer getPercentComments(){
    Integer result = 0;
    if( fNumLines > 0 ) {
      result = (100*fNumCommentLines)/fNumLines; 
    }
    return result;
  }
  
  /** Return true only if the file contains idenifier text configured in <tt>web.xml</tt> (java files only). */
  public Boolean isUnitTest(){
    return fHasUnitTestFingerprint;
  }
  
  /** Return the name and version of a .jar file's specification (.jar files only). */
  public String getSpecification(){
    return fSpecification;  
  }
  
  /** Return the number of tab characters (source files only).  */
  public Integer getNumTabs() {
    return fNumTabs;
  }
  
  /** Return the contents of the file as a single String (source files only). */
  public String getContent(){
    if( ! isSourceFile() ) {
      throw new IllegalArgumentException("Can return content only for source files. The extension of this file is not configured as being a source code file : " + getName());
    }
    StringBuilder result = new StringBuilder();
    try {
      Scanner scanner = new Scanner(fFile);
      while ( scanner.hasNextLine() ) {
        result.append(scanner.nextLine() + NEW_LINE);
      }
      scanner.close();
    }
    catch(FileNotFoundException ex){
      fLogger.severe("Cannot open file named " + fFile.getName());
    }
    return result.toString();
  }
  
  @Override public String toString(){
    //don't use ModelUtil, since can have large content.
    return fFile.getAbsolutePath();
  }
  
  @Override public boolean equals(Object aThat){
    Boolean result = ModelUtil.quickEquals(this, aThat);
    if ( result == null ) {
      FileInfo that = (FileInfo)aThat;
      result = ModelUtil.equalsFor(this.getSignificantFields(), that.getSignificantFields());
    }
    return result;
  }
  
  @Override public int hashCode(){
    if ( fHashCode == 0 ) {
      fHashCode = ModelUtil.hashCodeFor(getSignificantFields());
    }
    return fHashCode;
  }

  // PRIVATE //
  private final File fFile;
  private Integer fNumLines = 0;
  private Integer fNumCommentLines = 0;
  private Integer fNumTabs = 0;
  private Boolean fHasUnitTestFingerprint = false;
  private String fSpecification;
  private int fHashCode;
  
  private static final String SOURCE_FILE_EXTENSIONS = "SourceFileExtensions";
  private static List<String> fSourceFileExtensions = new ArrayList<String>();
  
  private static final String IMAGE_FILE_EXTENSIONS = "ImageFileExtensions";
  private static List<String> fImageFileExtensions = new ArrayList<String>();
  
  private static final String MARKUP_FILE_EXTENSIONS = "MarkupFileExtensions";
  private static List<String> fMarkupFileExtensions = new ArrayList<String>();
  
  private static final String IGNORABLE_FILES = "IgnorableFiles";
  private static List<String> fIgnorableFiles = new ArrayList<String>();
  
  private static final String UNIT_TEST_FINGERPRINT = "UnitTestFingerprint";
  private static String fUNIT_TEST_FINGERPRINT;

  private static final String SPECIFICATION_TITLE = "Specification-Title";
  private static final String SPECIFICATION_VERSION = "Specification-Version";
  
  private static final Logger fLogger = Util.getLogger(FileInfo.class);
  
  private Object[] getSignificantFields(){
    return new Object[] {fFile};
  }
  
  private static String fetchConfigFor(String aSetting, ServletConfig aConfig){
    String result = aConfig.getInitParameter(aSetting);
    Ensure.isPresentInWebXml(aSetting, result);
    fLogger.config("Config setting for " + aSetting + ": " + result);
    return result;
  }

  private static void setFileExtensions(String aRawConfig, List<String> aList){
    StringTokenizer parser = new StringTokenizer(aRawConfig, ",");
    while ( parser.hasMoreTokens() ) {
      String extension = parser.nextToken().trim();
      if( extension.startsWith(".")) {
        aList.add(extension);
      }
      else {
        fLogger.severe("File extension does not start with a '.' : " + Util.quote(extension));        
      }
    }
  }
  
  private static void setIgnorableFiles(String aRawConfig){
    StringTokenizer parser = new StringTokenizer(aRawConfig, ",");
    while ( parser.hasMoreTokens() ) {
      String ignorable = parser.nextToken().trim();
      if( Util.textHasContent(ignorable) ) {
        fIgnorableFiles.add(ignorable);
      }
      else {
        fLogger.severe("IgnorableFiles setting in web.xml not in expected form: " + Util.quote(aRawConfig));        
      }
    }
  }
  
  private void examineLines(){
    try {
      Scanner scanner = new Scanner(fFile);
      while ( scanner.hasNextLine() ) {
        String line = scanner.nextLine();
        ++fNumLines;
        fNumTabs = fNumTabs + numTabsIn(line);
        if ( isJavaSourceFile() ) {
          if ( isJavaComment(line) ) {
            ++fNumCommentLines;
          }
          if( hasUnitTestFingerprint(line) ){
            fHasUnitTestFingerprint = true;
          }
        }
      }
      scanner.close();
    }
    catch(FileNotFoundException ex){
      fLogger.severe("Cannot open file named " + fFile.getName());
    }
  }
  
  private boolean isJavaComment(String aLine){
    /*
     This is a block comment which would NOT be picked up by this method. 
     This kind of comment is rare in most java code, and is left out of 
     this implementation. If you wish to include this kind of comment, you 
     will need to amend this implementation.   
    */
    return aLine.trim().startsWith("/") || aLine.trim().startsWith("*");
  }
  
  private boolean hasUnitTestFingerprint(String aLine){
    return aLine.contains(fUNIT_TEST_FINGERPRINT);
  }
  
  private static String formattedList(List<String> aList){
    StringBuilder result = new StringBuilder();
    for (String item : aList){
      result.append(Util.quote(item) + " ");
    }
    return result.toString();
  }
  
  private void examineJarSpecs(){
    assert( isJarFile() );
    JarInputStream jarStream = null;
    try { 
      jarStream = new JarInputStream(new FileInputStream(fFile));
    }
    catch(IOException ex){
      fLogger.severe("Cannot open jar file, to fetch Specification-Version from the jar Manifest.");
    }
    fSpecification =  fetchSpecNamesAndVersions(jarStream);
  }
  
  /** 
  * A Jar can have more than one entry for spec name and version. 
  * For example, the servlet jar implements both servlet spec and jsp spec.
  * 
  * <P>Search for all attributes named Specification-Title and Specification-Version, 
  * and simply return them in the order they appear.
  * 
  * <P>If none of these appear, simply return an empty String.
  */
  private String fetchSpecNamesAndVersions(JarInputStream aJarStream) {
    StringBuilder result = new StringBuilder();
    Manifest manifest = aJarStream.getManifest();
    if ( manifest != null ){
      Attributes attrs = (Attributes)manifest.getMainAttributes();
      for (Iterator iter = attrs.keySet().iterator(); iter.hasNext(); ) {
        Attributes.Name attrName = (Attributes.Name)iter.next();
        addSpecAttrIfPresent(SPECIFICATION_TITLE, result, attrs, attrName);
        addSpecAttrIfPresent(SPECIFICATION_VERSION, result, attrs, attrName);
      }
      fLogger.fine("Specification-Version: " + result);
    }
    else {
      fLogger.fine("No manifest.");
    }
    return result.toString();
  }

  private void addSpecAttrIfPresent(String aName, StringBuilder aResult, Attributes aAttrs, Attributes.Name aAttrName) {
    if ( aName.equalsIgnoreCase(aAttrName.toString()) ) {
      if ( Util.textHasContent(aAttrs.getValue(aAttrName)) ){
        aResult.append(aAttrs.getValue(aAttrName) + " ");
      }
    }
  }
  
  private int numTabsIn(String aLine){
    int result = 0;
    StringCharacterIterator iterator = new StringCharacterIterator(aLine);
    char character =  iterator.current();
    while (character != CharacterIterator.DONE ){
      if( character == '\t') {
        result = result + 1;
      }
      character = iterator.next();      
    }
    return result;
  }
}