package hirondelle.web4jtools.metrics.files;

import java.util.*;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.util.Util;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4jtools.util.TemplatedPage;
import hirondelle.web4jtools.metrics.base.FileInfo;
import hirondelle.web4jtools.metrics.base.MetricsAction;
import hirondelle.web4jtools.metrics.base.BaseInfoAction;

/**
* Show a sortable file listing. 
* 
* <P>The file listing is not hierarchical. It is simply a list of all files in your project.
* The default sorting is by extension.
*/
public final class FilesAction extends MetricsAction {

  public FilesAction(RequestParser aRequestParser){
    super(FORWARD, aRequestParser);
  }

  /** Sort the files in the desired way, and present to the user.  */
  protected void calculateMetric() {
    sort(fFileList);
    //retain the sort by replacing the session scope object
    addToSession(BaseInfoAction.FILE_INFO_LIST_KEY, fFileList);
  }
  
  /** The column to sort by. See the <tt>sortLinks.tag</tt> file. */
  public static final RequestParameter SORT_ON = RequestParameter.withRegexCheck("SortOn", "(Extension|FullName|Lines|Modified|Bytes)");
  /** Ascending or descending order. See the <tt>sortLinks.tag</tt> file. */
  public static final RequestParameter ORDER = RequestParameter.withRegexCheck("Order", "(ASC|DESC)");
  
  // PRIVATE //
  private static final ResponsePage FORWARD = new ResponsePage("File Listing" , "view.jsp", FilesAction.class);
  
  private void sort(List<FileInfo> aFileInfos){
    String sortOn = getParamUnsafe(SORT_ON);
    String order = getParamUnsafe(ORDER);
    Collections.sort(aFileInfos, getComparator(sortOn));
    if( "DESC".equals(order) ) {
      Collections.reverse(aFileInfos);
    }
  }
  
  private Comparator<FileInfo> getComparator(String aSortOn){
    Comparator<FileInfo> result = null;
    //default sorting by extension
    if( "Extension".equals(aSortOn) || ! Util.textHasContent(aSortOn) ) {
      result = new Comparator<FileInfo>() {
        public int compare(FileInfo aThis, FileInfo aThat) {
          return aThis.getExtension().compareTo(aThat.getExtension());
        };
      };
    }
    else if ( "FullName".equals(aSortOn) ) {
      result = new Comparator<FileInfo>() {
        public int compare(FileInfo aThis, FileInfo aThat) {
          return aThis.getName().compareTo(aThat.getName());
        };
      };
    }
    else if ( "Lines".equals(aSortOn) ) {
      result = new Comparator<FileInfo>() {
        public int compare(FileInfo aThis, FileInfo aThat) {
          return aThis.getNumLines().compareTo(aThat.getNumLines());
        };
      };
    }
    else if ( "Modified".equals(aSortOn) ) {
      result = new Comparator<FileInfo>() {
        public int compare(FileInfo aThis, FileInfo aThat) {
          return aThis.getLastModified().compareTo(aThat.getLastModified());
        };
      };
    }
    else if ( "Bytes".equals(aSortOn) ) {
      result = new Comparator<FileInfo>() {
        public int compare(FileInfo aThis, FileInfo aThat) {
          return aThis.getSize().compareTo(aThat.getSize());
        };
      };
    }
    else {
      throw new AssertionError("Unknown sort column : " + Util.quote(aSortOn));
    }
    return result;
  }
}
