package hirondelle.web4jtools.metrics.lines;

import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.util.Util;
import hirondelle.web4jtools.metrics.base.FileInfo;
import hirondelle.web4jtools.metrics.base.MetricsAction;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
* Display number of lines of source code, grouped by file extension. 
*/
public final class LinesAction extends MetricsAction {

  /** Full constructor. */
  public LinesAction(RequestParser aRequestParser){
    super(FORWARD, aRequestParser);
  }

  /** List the number of lines of source code, per file extension.  Default sort by total lines.*/
  protected void calculateMetric()  {
    LinesCounter linesCounter = getLinesCounter();
    List<LineStat> stats = linesCounter.getStats();
    sort(stats);
    addToRequest("stats", stats);
    addToRequest("totalLines", linesCounter.getTotalLines());
    addToRequest("totalFiles", linesCounter.getTotalFiles());
  }

  /** Column to sort by. See the <tt>sortLinks.tag</tt> file. */
  public static final RequestParameter SORT_ON = RequestParameter.withRegexCheck("SortOn", "(Extension|AvgLines|NumFiles|TotalLines)");
  /** Ascending or descending order. See the <tt>sortLinks.tag</tt> file. */
  public static final RequestParameter ORDER = RequestParameter.withRegexCheck("Order", "(ASC|DESC)");
  
  // PRIVATE //
  private static final ResponsePage FORWARD = new ResponsePage("Lines By File Extension", "view.jsp", LinesAction.class);
  
  private LinesCounter getLinesCounter(){
    LinesCounter counter = new LinesCounter();
    for(FileInfo fileInfo: fFileList){
      if ( fileInfo.isSourceFile() ) {
        counter.add(fileInfo);
      }
    }
    return counter;    
  }

  private void sort(List<LineStat> aStats){
    String sortOn = getParamUnsafe(SORT_ON);
    String order = getParamUnsafe(ORDER);
    Collections.sort(aStats, getComparator(sortOn));
    if( "DESC".equals(order) || ! Util.textHasContent(order)) {
      Collections.reverse(aStats);
    }
  }
  
  private Comparator<LineStat> getComparator(String aSortOn){
    Comparator<LineStat> result = null;
    if( "Extension".equals(aSortOn)  ) {
      result = new Comparator<LineStat>() {
        public int compare(LineStat aThis, LineStat aThat) {
          return aThis.getExtension().compareTo(aThat.getExtension());
        };
      };
    }
    else if ( "AvgLines".equals(aSortOn) ) {
      result = new Comparator<LineStat>() {
        public int compare(LineStat aThis, LineStat aThat) {
          return aThis.getAverageLinesPerFile().compareTo(aThat.getAverageLinesPerFile());
        };
      };
      
    }
    else if ( "NumFiles".equals(aSortOn) ) {
      result = new Comparator<LineStat>() {
        public int compare(LineStat aThis, LineStat aThat) {
          return aThis.getNumFiles().compareTo(aThat.getNumFiles());
        };
      };
      
    }
    //default sorting by total lines
    else if ( "TotalLines".equals(aSortOn) || ! Util.textHasContent(aSortOn) ) {
      result = new Comparator<LineStat>() {
        public int compare(LineStat aThis, LineStat aThat) {
          return aThis.getNumLines().compareTo(aThat.getNumLines());
        };
      };
    }
    else {
      throw new AssertionError("Unknown sort column : " + Util.quote(aSortOn));
    }
    return result;
  }
}
