package hirondelle.web4jtools.util;

import hirondelle.web4j.util.Util;
import hirondelle.web4jtools.codegenerator.codes.ControlStyle;
import hirondelle.web4jtools.codegenerator.codes.FieldType;
import hirondelle.web4jtools.codegenerator.field.Field;
import hirondelle.web4j.util.Consts;

/**
* Utility functions called from JSPs, using JSTL functions.
* 
* <P>Most of these functions are related to code generation. If the formatting provided by 
* this class is not adequate for your needs, you will need to change its implementation.
* 
* <P>These items are referenced in <tt>web4j.tld</tt>. 
* Since these functions are mostly used in pages served as plain text, it is appropriate that they 
* return <tt>String</tt> instead of <tt>SafeText</tt>.
* 
* <P>The text passed as incoming argument to these methods should be in a 
* 'natural text' style, as in <tt>'Jet Engine'</tt>, and not formatted in any 
* special way (such as <tt>'JetEngine'</tt> or <tt>'JET_ENGINE'</tt>).
*/
public final class Functions {

  /**
  * Format as a Java constant : <tt>LIKE_THIS</tt>.
  * 
  * <P>Any spaces are replaced with an underscore, and the text is converted to upper case.
  */
  public static String asConstant(String aFeatureName){
    String result = aFeatureName.replace(SPACE, UNDERSCORE);
    return result.toUpperCase();
  }

  /**
  * Format as a Java class name identifier : <tt>LikeThis</tt>.
  * 
  * <P>Any spaces are removed. 
  */
  public static String asType(String aText){
    return Util.withNoSpaces(aText);
  }
  
  /** Format as an argument to a method or a constructor:  <tt>aLikeThis</tt>. */
  public static String asArgument(String aFieldName){
    return "a" + asType(aFieldName);
  }
  
  /** Format as an object field reference: <tt>fLikeThis</tt>. */
  public static String asField(String aFieldName){
    return "f" + asType(aFieldName);
  }
  
  /** Format as a table or column name in an SQL statement: <tt>LikeThis</tt>. */
  public static String asDbIdentifier(String aFieldName){
    //some might prefer this as upper case.
    return asType(aFieldName);
  }
  
  /** Format as a JSP Expression Language reference to an object or a property: <tt>likeThis</tt>.  */
  public static String asELIdentifier(String aFieldName) {
    return firstLetterLowerCase(asType(aFieldName));
  }
  
  /** Format as a local object in the body of a method: <tt>likeThis</tt>.  */
  public static String asLocal(String aName){
    return Util.withNoSpaces(firstLetterLowerCase(aName));
  }
  
  /** Present a field as a complete HTML form control.   */
  public static String asControl(Field aField){
    String result = Consts.EMPTY_STRING;
    String name = asType(aField.getName());
    String type = aField.getControlStyle().getControlName();
    ControlStyle style = aField.getControlStyle();
    if ( ControlStyle.Text == style || 
         ControlStyle.Hidden == style || 
         ControlStyle.Password == style
    ) {
      result = "<input type='" + type + "' name='" + name + "'>";
    }
    else if ( ControlStyle.Checkbox == style ) {
      result = "<input type='" + type+ "' name='" + name + "' value='true'>";
    }
    else if ( ControlStyle.Textarea == style ){
      result = "<textarea name='" + name + "'></textarea>";
    }
    else if (ControlStyle.Radio == style ) {
      result = "<c:forEach var='item' items='${" + aField.getCodeTable() + "}'><input name='" + name + "' type='radio' value='${item}'> ${item} </c:forEach>";
    }
    else if ( ControlStyle.Select == style ){
      result = "<select name='" + name +  "'> <option> </option> <c:forEach var='item' items='${" + aField.getCodeTable() + "}'><option value='${item}'>${item}</option></c:forEach></select>";
    }
    return result;
  }
  
  /** 
  * Generate text snippet used for performing validations with the <tt>Check</tt> class.
  * 
  * An example return value : 
  * <PRE>'Check.range(1, 100), Check.isRegex()'</PRE>  
  */
  public static String checks(Field aField) {
    StringBuilder result = new StringBuilder();
    //range, min, max
    if( aField.getMinimum() != null && aField.getMaximum() != null ) {
      result.append("Check.range(" + getRange(aField) + "), ");
    }
    else if( aField.getMinimum() != null ) {
      result.append("Check.min(" + getMin(aField) + "), ");
    }
    else if ( aField.getMaximum() != null ) {
      result.append("Check.max(" + getMax(aField) + "), ");
    }
    
    if( aField.getNumDecimals() != null ) {
      result.append("Check.numDecimalsAlways(" + aField.getNumDecimals() + "), ");
    }
    
    if( aField.getCheckPattern() != null ){
      result.append("Check.isRegex(), ");
    }
    
    if ( aField.getCheckEmail() == true ){
      result.append("Check.isEmail(), ");
    }
    
    if( aField.getCheckSpam() == true ){
      result.append("Check.isSpam(), ");
    }
    return result.toString();
  }
  
  /**  Return a file:// URI for the given file.  */
  public static String fileURL(String aFileName){
    return "file:///" + aFileName.replace('\\', '/');
  }
  
  // PRIVATE //
  private static final char SPACE = ' ';
  private static final char UNDERSCORE = '_';
  
  //prevent construction by the caller
  private Functions() {}
  
  private static String firstLetterLowerCase(String aIdentifier){
    String firstLetter = aIdentifier.substring(0,1);
    return aIdentifier.replaceFirst(firstLetter, firstLetter.toLowerCase());
  }
  
  private static String getMin(Field aField){
    String result = aField.getMinimum().toString(); //default
    if ( FieldType.BigDecimal == aField.getType()  ) {
      result = "new BigDecimal(\"" + aField.getMinimum() + "\")";      
    }
    else if( FieldType.Decimal == aField.getType() ) {
      result = "Decimal.from(\"" + aField.getMinimum() + "\")";      
    }
    return result;
  }
  
  private static String getMax(Field aField){
    String result = aField.getMaximum().toString(); //default
    if ( FieldType.BigDecimal == aField.getType()  ) {
      result = "new BigDecimal(\"" + aField.getMaximum() + "\")";      
    }
    else if( FieldType.Decimal == aField.getType() ) {
      result = "Decimal.from(\"" + aField.getMaximum() + "\")";      
    }
    return result;
  }
  
  private static String getRange(Field aField){
    return getMin(aField) + ", " + getMax(aField);
  }
}
