package hirondelle.web4jtools.util;

import java.util.logging.*;
import javax.servlet.jsp.JspException;
import hirondelle.web4j.ui.tag.TagHelper;
import hirondelle.web4j.util.Util;
import hirondelle.web4jtools.codegenerator.feature.Feature;
import hirondelle.web4jtools.codegenerator.feature.FeatureAction;

import java.io.*;
import hirondelle.web4j.util.Consts;
import javax.servlet.ServletConfig;
import hirondelle.web4jtools.util.Ensure;

/** 
* Echo the trimmed body, while also saving it as a file on the server as a side-effect.
* 
* <P>Uses two settings in <tt>web.xml</tt> : 
* <ul>
* <li><tt>BaseDirectoryForGeneratedFiles</tt> - the base or root directory for your classes. 
* This code generator follows the 
* <a href='http://www.web4j.com/UserGuide.jsp#DirectoryStructure'>package-by-feature</a> style 
* recommended by WEB4J, in which all items related to a given feature are placed in a single directory: 
* JSP, Action, Model, DAO, and .sql file. A value of <tt>'Disabled'</tt> for this setting will turn off 
* all file generation. When enabled, the value has the form <tt>[project-root]/WEB-INF/classes/</tt>.
* <li><tt>CharacterSetForGeneratedFiles</tt> - controls the character encoding of generated files.
* </ul>
* 
* <P>Example use case in a JSP :
* <PRE>
&lt;w:saveToFile named='statements.sql'&gt;
 ...JSP content...
&lt;/w:saveToFile&gt;
* </PRE>
* The exact location of the generated files depends on the package name. If the above snippet is 
* for a feature having the package <tt>com.xyz.jet</tt>, then the generated file will be 
* <P><tt>[project-root]/WEB-INF/classes/com/xyz/jet/statements.sql</tt>
*/
public final class SaveToFile extends TagHelper {

  /** Read in configuration from <tt>web.xml</tt>  */
  static public void readConfig(ServletConfig aConfig){
    fBASE_DIR = aConfig.getInitParameter(BASE_DIR);
    fCHARSET = aConfig.getInitParameter(CHARSET);
    Ensure.isPresentInWebXml(BASE_DIR, fBASE_DIR);
    Ensure.isPresentInWebXml(CHARSET, fCHARSET);
  }
  
  /** Set the name of the emitted file.  */
  public void setNamed(String aFileName){
    fFileName = aFileName;
  }
  
  @Override protected String getEmittedText(String aBody) throws JspException, IOException {
    if( isEnabled() ) {
      saveToFile(aBody.trim(), getOutputFileName());
    }
    else {
      fLogger.fine("Not saving to file, since disabled.");
    }
    return aBody;
  }
  
  /** 
  * Return <tt>true</tt> only if <tt>BaseDirectoryForGeneratedFiles</tt> (<tt>web.xml</tt>) is not 
  * set to <tt>'Disabled'</tt> (ignoring case). 
  */
  public static boolean isEnabled(){
    return ! DISABLED.equalsIgnoreCase(fBASE_DIR);  
  }
  
  // PRIVATE //
  private String fFileName;
  
  private static final String BASE_DIR = "BaseDirectoryForGeneratedFiles";
  private static final String DISABLED = "Disabled";
  private static String fBASE_DIR;
  
  private static final String CHARSET = "CharacterSetForGeneratedFiles"; 
  private static String fCHARSET;
  
  private static final Logger fLogger = Util.getLogger(SaveToFile.class);
  
  private String getOutputFileName() throws IOException {
    String directory = getProjectRootForClasses() + getPackagePath();
    ensureDirectoryExists(directory);
    ensureFileExists();
    String result = directory + fFileName;
    fLogger.fine("Saving to file named : " + result);
    return result;
  }
  
  private String getProjectRootForClasses(){
    return fBASE_DIR;
  }
  
  private String getPackagePath(){
    Feature feature = (Feature)getRequest().getSession().getAttribute(FeatureAction.FEATURE_KEY);
    return addDirectorySeparators(feature.getPackageName());
  }

  private String addDirectorySeparators(String aPackageName){
    return Util.replace(aPackageName, ".", Consts.FILE_SEPARATOR) + Consts.FILE_SEPARATOR;
  }
  
  /** Emit the text into the file.  */
  private void saveToFile(String aBody, String aOutputFileName) throws IOException {
    Writer out = null;
    try {
        FileOutputStream fos = new FileOutputStream(aOutputFileName);
        out = new BufferedWriter(new OutputStreamWriter(fos, fCHARSET));
        out.write(aBody);
    }
    finally {
      if ( out != null) out.close();
    }
  }
  
  private void ensureDirectoryExists(String aDirName){
    File dir = new File(aDirName);
    dir.mkdirs();
  }
  
  private void ensureFileExists() throws IOException {
    File file = new File(fFileName);
    file.createNewFile(); //does not create if already exists
  }
}
